/* Main processing code for Services. */

#include "services.h"

/*************************************************************************/

/* Split a buffer into argc/argv-like things.  Return -1 on error.
 * Destroys buf by side-effect.  If (colon_special), then treat a parameter
 * with a leading ':' as the last parameter of the line, per the IRC RFC. */
int split_buf(char *buf, char ***argv, int colon_special)
{
    int argvsize = 8;
    int argc;
    char *s, *t;

    if (!(*argv = malloc(sizeof(char *) * argvsize)))
	return -1;
    argc = 0;
    while (*buf) {
	if (argc == argvsize) {
	    argvsize += 8;
	    if (!(*argv = realloc(*argv, sizeof(char *) * argvsize)))
		return -1;
	}
	if (*buf == ':') {
	    (*argv)[argc++] = buf+1;
	    buf = "";
	} else {
	    s = strpbrk(buf, " ");
	    if (s) {
		*s = 0;
		while (isspace(*++s))
		    ;
	    } else {
		s = buf + strlen(buf);
	    }
	    (*argv)[argc++] = buf;
	    buf = s;
	}
    }
    return argc;
}

/*************************************************************************/

/* Main processing routine. */
void process()
{
    char source[64];
    char cmd[64];
    char buf[512];	/* Longest legal IRC command line */
    char *s;
    int ac;		/* Parameters for the command */
    char **av;


    /* First make a copy of the buffer so we have the original ico crash. */
    strscpy(buf, inbuf, sizeof(buf));

/*log("inbuf: %s", inbuf);*/

    /* Split the buffer into pieces. */
    if (*buf == ':') {
	s = strpbrk(buf, " ");
	if (!s)
	    return;
	*s = 0;
	while (isspace(*++s))
	    ;
	strscpy(source, buf+1, sizeof(source));
	strcpy(buf, s);
    } else {
	*source = 0;
    }
    s = strpbrk(buf, " ");
    if (!s)
	return;
    *s = 0;
    while (isspace(*++s))
	;
    strscpy(cmd, buf, sizeof(cmd));
    ac = split_buf(s, &av, 1);


    if (stricmp(cmd, "PING") == 0) {

	if (ac < 1)
	    return;
	send_cmd(SERVER_NAME, "PONG %s", av[0]);

    } else if (stricmp(cmd, "AWAY") == 0) {

	/* Do nothing */

    } else if (stricmp(cmd, "JOIN") == 0) {

	if (ac < 1)
	    return;
	do_join(source, ac, av);

    } else if (stricmp(cmd, "KICK") == 0) {

	if (ac < 3)
	    return;
	do_kick(source, ac, av);

    } else if (stricmp(cmd, "KILL") == 0) {

	if (ac < 2)
	    return;
	do_kill(source, ac, av);

    } else if (stricmp(cmd, "MODE") == 0) {

	if (ac < 2)
	    return;
	if (*av[0] == '#' || *av[0] == '&')
	    do_cmode(source, ac, av);
	else
	    do_umode(source, ac, av);

    } else if (stricmp(cmd, "NICK") == 0) {

	if ((!*source && ac < 7) || (*source && ac < 2))
	    return;
	do_nick(source, ac, av);

    } else if (stricmp(cmd, "PART") == 0) {

	if (ac < 1)
	    return;
	do_part(source, ac, av);

    } else if (stricmp(cmd, "PASS") == 0) {

	/* Do nothing */

    } else if (stricmp(cmd, "PRIVMSG") == 0 || stricmp(cmd, "NOTICE") == 0) {

	if (ac < 2)
	    return;
	if (stricmp(av[0], s_OperServ) == 0) {
	    if (is_oper(source))
		operserv(source, av[1]);
	    else
		notice(s_OperServ, source, "Access denied.");
	} else if (stricmp(av[0], s_NickServ) == 0) {
	    nickserv(source, av[1]);
	} else if (stricmp(av[0], s_ChanServ) == 0) {
	    chanserv(source, av[1]);
	}

    } else if (stricmp(cmd, "QUIT") == 0) {

	if (ac < 1)
	    return;
	do_quit(source, ac, av);

    } else if (stricmp(cmd, "SERVER") == 0) {

	/* Do nothing */

    } else if (stricmp(cmd, "SQUIT") == 0) {

	/* Do nothing */

    } else if (stricmp(cmd, "TOPIC") == 0) {

	if (ac < 3)
	    return;
	do_topic(source, ac, av);

    } else if (stricmp(cmd, "USER") == 0) {

	/* Do nothing */

    } else {

	log("unknown message from server (%s)", inbuf);

    }

}
